# Volatility
#
# Authors:
# Michael Hale Ligh <michael@memoryanalysis.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import volatility.obj as obj
import volatility.utils as utils
import volatility.plugins.taskmods as taskmods

#--------------------------------------------------------------------------------
# object classes 
#--------------------------------------------------------------------------------   

class _HEAP(obj.CType):
    """ A Heap on XP and 2003 """
        
    def is_valid(self):
        return obj.CType.is_valid(self) and self.Signature == 0xeeffeeff

    def segments(self):
        """ A list of the _HEAP_SEGMENTs. 

        This is an array of pointers so we have to deref
        before returning or the caller will be calling 
        is_valid on the pointer and not the object. 
        """
        return [seg.dereference() for seg in self.Segments if seg != 0]

class _HEAP_SEGMENT(obj.CType):
    """ A Heap Segment on XP and 2003 """    

    def is_valid(self):
        return obj.CType.is_valid(self) and self.Signature == 0xffeeffee

    def heap_entries(self):
        """Enumerate the heaps in this segment. 

        ##FIXME: 
        * Raise ValueError if corruptions are detected. 
        * Should we start at FirstEntry or Entry?
        """

        next = self.Entry #FirstEntry.dereference()
        last = self.LastValidEntry.dereference()

        chunk_size = self.obj_vm.profile.get_obj_size("_HEAP_ENTRY")

        while (next and 
                    next.obj_offset < last.obj_offset):

            yield next

            next = obj.Object("_HEAP_ENTRY", 
                    offset = next.obj_offset + next.Size * chunk_size, 
                    vm = next.obj_vm)

class _HEAP_ENTRY(obj.CType):
    """ A Heap Entry """

    def get_data(self):

        chunk_size = self.obj_vm.profile.get_obj_size("_HEAP_ENTRY")

        return self.obj_vm.zread(
                    self.obj_offset + chunk_size, 
                    self.Size * chunk_size
                    )

    def get_extra(self):

        chunk_size = self.obj_vm.profile.get_obj_size("_HEAP_ENTRY")

        return obj.Object("_HEAP_ENTRY_EXTRA", 
                    offset = self.obj_offset + (chunk_size * (self.Size - 1)), 
                    vm = self.obj_vm)

#--------------------------------------------------------------------------------
# profile modifications
#--------------------------------------------------------------------------------   

class XPHeapModification(obj.ProfileModification):

    before = ["WindowsObjectClasses"]

    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x : x == 5, 
                  'memory_model' : lambda x : x == '32bit'}

    def modification(self, profile):

        heap_flags = {
            'HEAP_NO_SERIALIZE': 0, 
            'HEAP_GROWABLE': 1, 
            'HEAP_GENERATE_EXCEPTIONS': 2, 
            'HEAP_ZERO_MEMORY': 3, 
            'HEAP_REALLOC_IN_PLACE_ONLY': 4, 
            'HEAP_TAIL_CHECKING_ENABLED': 5, 
            'HEAP_FREE_CHECKING_ENABLED': 6, 
            'HEAP_DISABLE_COALESCE_ON_FREE': 7, 
            'HEAP_SETTABLE_USER_VALUE': 8,             
            'HEAP_CREATE_ALIGN_16': 16, 
            'HEAP_CREATE_ENABLE_TRACING': 17, 
            'HEAP_CREATE_ENABLE_EXECUTE': 18, 
            'HEAP_FLAG_PAGE_ALLOCS': 24, 
            'HEAP_PROTECTION_ENABLED': 25, 
            'HEAP_CAPTURE_STACK_BACKTRACES': 27, 
            'HEAP_SKIP_VALIDATION_CHECKS': 28, 
            'HEAP_VALIDATE_ALL_ENABLED': 29, 
            'HEAP_VALIDATE_PARAMETERS_ENABLED': 30, 
            'HEAP_LOCK_USER_ALLOCATED': 31, 
            }

        entry_flags = {
            #'HEAP_ENTRY_BUSY': 0, 
            "busy": 0, 
            #'HEAP_ENTRY_EXTRA_PRESENT': 1,   
            "extra": 1,
            #'HEAP_ENTRY_FILL_PATTERN': 2, 
            "fill": 2, 
            #'HEAP_ENTRY_VIRTUAL_ALLOC': 3, 
            "virtual": 3, 
            #'HEAP_ENTRY_LAST_ENTRY': 4, 
            "last": 4, 
            #'HEAP_ENTRY_SETTABLE_FLAG1': 5, 
            "flag1": 5,
            #'HEAP_ENTRY_SETTABLE_FLAG2': 6, 
            "flag2": 6, 
            #'HEAP_ENTRY_SETTABLE_FLAG3': 7
            "flag3": 7
            }

        profile.merge_overlay({
            '_HEAP': [ None, { 
            'Flags': [ None, ['Flags', {'bitmap': heap_flags}]],
            'ForceFlags': [ None, ['Flags', {'bitmap': heap_flags}]],
            }], 
            '_HEAP_FREE_ENTRY': [ None, { 
            'Flags': [ None, ['Flags', {'target': 'unsigned char', 'bitmap': entry_flags}]],
            }], 
            '_HEAP_ENTRY': [ None, { 
            'Flags': [ None, ['Flags', {'target': 'unsigned char', 'bitmap': entry_flags}]],
            }], 
            '_HEAP_SEGMENT': [ None, { 
            'Flags': [ None, ['Flags', {'bitmap': {'HEAP_USER_ALLOCATED': 0}}]],
            }],
        })

        profile.object_classes.update({
            '_HEAP_ENTRY': _HEAP_ENTRY, 
            '_HEAP': _HEAP, 
            '_HEAP_SEGMENT': _HEAP_SEGMENT, 
        })

class Notepad(taskmods.DllList):
    """List currently displayed notepad text"""

    def __init__(self, config, *args, **kwargs):
        taskmods.DllList.__init__(self, config, *args, **kwargs)
        config.add_option("DUMP-DIR", short_option = "D", default = None, 
            help = "Dump binary data to this directory")

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('major', 0) == 5)

    def render_text(self, outfd, data):
        for task in data:

            # only looking for notepad
            if str(task.ImageFileName).lower() != "notepad.exe":
                continue
            outfd.write("Process: {0}\n".format(task.UniqueProcessId))

            entry_size = task.obj_vm.profile.get_obj_size("_HEAP_ENTRY")
            heap = task.Peb.ProcessHeap.dereference_as("_HEAP")

            for segment in heap.segments():
                for entry in segment.heap_entries():
                
                    # the extra heap data is present 
                    if "extra" not in str(entry.Flags):
                        continue 

                    text = obj.Object("String", 
                                     offset = entry.obj_offset + entry_size, 
                                     vm = task.get_process_address_space(),
                                     length = entry.Size * entry_size, 
                                     encoding = "utf16")

                    if not text or len(text) == 0:
                        continue 

                    if self._config.DUMP_DIR:
                        name = "notepad.{0}.txt".format(task.UniqueProcessId)
                        path = os.path.join(self._config.DUMP_DIR, name)
                        with open(path, "wb") as handle:
                            handle.write(entry.get_data())
                        outfd.write("Dumped To: {0}\n".format(path))

                    outfd.write("Text:\n{0}\n\n".format(text))


# Volatility
# Copyright (C) 2007,2008 Volatile Systems
# Copyright (C) 2009 Timothy D. Morgan (strings optimization)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from bisect import bisect_right
import volatility.plugins.mac.pstasks as mac_tasks
import volatility.plugins.strings as strings
import volatility.plugins.mac.common as mac_common
import volatility.plugins.mac.lsmod as mac_lsmod

class mac_strings(strings.Strings, mac_common.AbstractMacCommand):
    """Match physical offsets to virtual addresses (may take a while, VERY verbose)"""

    @staticmethod
    def is_valid_profile(profile):
        return profile.metadata.get('os', 'Unknown').lower() == 'mac'

    def get_processes(self, addr_space):
        """Enumerate processes based on user options.

        :param      addr_space | <addrspace.AbstractVirtualAddressSpace>

        :returns    <list> 
        """
       
        tasks = mac_tasks.mac_tasks(self._config).calculate()

        try:
            if self._config.PID is not None:
                pidlist = [int(p) for p in self._config.PID.split(',')]
                tasks = [t for t in tasks if int(t.p_pid) in pidlist]
        except (ValueError, TypeError):
            debug.error("Invalid PID {0}".format(self._config.PID))

        return tasks

    @classmethod
    def get_modules(cls, addr_space):    
        """Enumerate the kernel modules. 

        :param      addr_space | <addrspace.AbstractVirtualAddressSpace>
        
        :returns    <tuple>
        """

        mask = addr_space.address_mask
        config = addr_space.get_config()
        modules = mac_lsmod.mac_lsmod(config).calculate()
        mods = dict((mask(mod.address), mod) for mod in modules)
        mod_addrs = sorted(mods.keys())
         
        return (mods, mod_addrs)

    @classmethod
    def find_module(cls, modlist, mod_addrs, addr_space, vpage):
        """Determine which module owns a virtual page. 

        :param      modlist     | <list>
                    mod_addrs   | <list>
                    addr_space  | <addrspace.AbstractVirtualAddressSpace>
                    vpage       | <int> 
        
        :returns    <module> || None
        """

        pos = bisect_right(mod_addrs, vpage) - 1
        if pos == -1:
            return None
        mod = modlist[mod_addrs[pos]]

        compare = mod.obj_vm.address_compare
        if (compare(vpage, mod.address) != -1 and
                compare(vpage, mod.address + mod.m('size')) == -1):
            return mod
        else:
            return None

    @classmethod
    def get_module_name(cls, module):
        """Get the name of a kernel module.

        :param      module      | <module>
        
        :returns    <str>
        """

        return str(module.m("name"))

    @classmethod
    def get_task_pid(cls, task):
        """Get the PID of a process. 

        :param      task   | <task>
        
        :returns    <int>
        """
        return task.p_pid
 



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import struct
import volatility.obj as obj
import volatility.addrspace as addrspace
import volatility.constants as constants
import volatility.utils as utils
import volatility.plugins.overlays.windows.win8 as win8
import volatility.plugins.patchguard as patchguard

try:
    import distorm3
    has_distorm = True
except ImportError:
    has_distorm = False

class VolatilityKDBG(obj.VolatilityMagic):
    """A Scanner for KDBG data within an address space. 

    This implementation is specific for Windows 8 / 2012 
    64-bit versions because the KDBG block is encoded. We 
    have to find it a special way and then perform the 
    decoding routine before Volatility plugins can run. 
    """

    def v(self):
        """The --kdbg parameter for this Win8/2012 KDBG 
        structure is the virtual address of the 
        nt!KdCopyDataBlock function (see kdbgscan output).
        """

        if self.value is None:
            return self.get_best_suggestion()
        else:
            return self.copy_data_block(self.value)

    def get_suggestions(self):
        if self.value:
            yield self.copy_data_block(self.value)
        for x in self.generate_suggestions():
            yield x

    def decode_kdbg(self, vals):
        """Decoder the KDBG block using the provided 
        magic values and the algorithm reversed from 
        the Windows kernel file."""

        block_encoded, kdbg_block, wait_never, wait_always = vals
        header = obj.VolMagic(self.obj_vm).KDBGHeader.v()
        kdbg_size = struct.unpack("<H", header[-2:])[0]
        buffer = ""

        entries = obj.Object("Array", 
                            targetType = "unsigned long long", 
                            count = kdbg_size / 8, 
                            offset = kdbg_block, vm = self.obj_vm)

        for entry in entries: 
            low_byte = (wait_never & 0xFFFFFFFF) & 0xFF
            entry = patchguard.rol(entry ^ wait_never, low_byte)
            swap_xor = block_encoded.obj_offset | 0xFFFF000000000000
            entry = patchguard.bswap(entry ^ swap_xor)
            buffer += struct.pack("Q", entry ^ wait_always) 

        return buffer

    def copy_data_block(self, full_addr):
        """This function emulates nt!KdCopyDataBlock on a live 
        machine by finding the encoded KDBG structure and using
        the required entropy values to decode it."""

        # this unpacks the kdbgsize from the signature 
        header = obj.VolMagic(self.obj_vm).KDBGHeader.v()
        kdbg_size = struct.unpack("<H", header[-2:])[0]

        size_str = struct.pack("I", kdbg_size)
        alignment = 8 
        addr_space = self.obj_vm
        bits = distorm3.Decode64Bits

        # nt!KdCopyDataBlock is about 100 bytes, we don't want to read
        # too little and truncate the function, but too much will reach
        # into other function's space
        code = addr_space.read(full_addr, 300)

        # potentially we crossed a boundary into swapped or unallocated space
        if code == None:
            return obj.NoneObject("Crossed a code boundary")

        if code.find(struct.pack("I", kdbg_size / alignment)) == -1:
            return obj.NoneObject("Cannot find KDBG size signature")

        version = (addr_space.profile.metadata.get('major', 0), addr_space.profile.metadata.get('minor', 0))
        if version < (6, 4):
            # we don't perform this check for Windows 10.x
            if code.find(size_str) == -1:
                return obj.NoneObject("Cannot find KDBG size signature")  

        ops = list(distorm3.Decompose(full_addr, code, bits))

        # nt!KdDebuggerDataBlock
        kdbg_block = None
        # nt!KiWaitNever
        wait_never = None
        # nt!KiWaitAlways
        wait_always = None
        # nt!KdpDataBlockEncoded
        block_encoded = None

        for op in ops:
            # cmp cs:KdpDataBlockEncoded, 0
            if (not block_encoded and op.mnemonic == "CMP" and 
                        op.operands[0].type == "AbsoluteMemory" and 
                        op.operands[1].type == "Immediate" and 
                        op.operands[1].value == 0):
                # an x64 RIP turned absolute 
                offset = op.address + op.size + op.operands[0].disp
                block_encoded = obj.Object("unsigned char", 
                                        offset = offset,
                                        vm = addr_space)
            # lea rdx, KdDebuggerDataBlock
            elif (not kdbg_block and op.mnemonic == "LEA" and 
                        op.operands[0].type == "Register" and 
                        op.operands[0].size == 64 and 
                        op.operands[1].type == "AbsoluteMemory" and 
                        op.operands[1].dispSize == 32):
                kdbg_block = op.address + op.size + op.operands[1].disp 
            # mov r10, cs:KiWaitNever
            elif (not wait_never and op.mnemonic == "MOV" and 
                        op.operands[0].type == "Register" and 
                        op.operands[0].size == 64 and 
                        op.operands[1].type == "AbsoluteMemory" and 
                        op.operands[1].dispSize == 32):
                offset = op.address + op.size + op.operands[1].disp
                wait_never = obj.Object("unsigned long long", 
                                        offset = offset, 
                                        vm = addr_space)
            # mov r11, cs:KiWaitAlways (Win 8 x64)
            # xor rdx, cs:KiWaitAlways (Win 8.1 x64)
            elif (not wait_always and op.mnemonic in ["MOV", "XOR"] and 
                        op.operands[0].type == "Register" and 
                        op.operands[0].size == 64 and 
                        op.operands[1].type == "AbsoluteMemory" and 
                        op.operands[1].dispSize == 32):
                offset = op.address + op.size + op.operands[1].disp
                wait_always = obj.Object("unsigned long long", 
                                        offset = offset,
                                        vm = addr_space)
                break
            elif op.mnemonic == "RET":
                break

        # check if we've found all the required offsets 
        if (block_encoded != None 
                    and kdbg_block != None 
                    and wait_never != None 
                    and wait_always != None):
            
            # some acquisition tools decode the KDBG block but leave 
            # nt!KdpDataBlockEncoded set, so we handle it here. 
            tag_offset = addr_space.profile.get_obj_offset("_DBGKD_DEBUG_DATA_HEADER64", "OwnerTag")
            signature = addr_space.read(kdbg_block + tag_offset, 4)

            if block_encoded == 1 and signature != "KDBG":
                vals = block_encoded, kdbg_block, wait_never, wait_always
                data = self.decode_kdbg(vals)
                buff = addrspace.BufferAddressSpace(
                            config = addr_space.get_config(),
                            base_offset = kdbg_block,
                            data = data)
                kdbg = obj.Object("_KDDEBUGGER_DATA64", 
                            offset = kdbg_block, 
                            vm = buff, 
                            native_vm = addr_space)
            else:
                kdbg = obj.Object("_KDDEBUGGER_DATA64", 
                            offset = kdbg_block, 
                            vm = addr_space)

            kdbg.newattr('KdCopyDataBlock', full_addr)
            kdbg.newattr('block_encoded', block_encoded == 1 and signature != "KDBG")
            kdbg.newattr('wait_never', wait_never)
            kdbg.newattr('wait_always', wait_always)                    

            return kdbg
        else:
            return obj.NoneObject("Cannot find decoding entropy values")

    def generate_suggestions(self):
        """Generates a list of possible KDBG structure locations"""

        if not has_distorm:
            raise StopIteration("The distorm3 Python library is required")

        overlap = 20
        offset = 0 
        current_offset = offset
        addr_space = self.obj_vm
        
        addresses = sorted(addr_space.get_available_addresses())
        for (range_start, range_size) in addresses:
            # Jump to the next available point to scan from
            current_offset = max(range_start, current_offset)
            range_end = range_start + range_size

            while (current_offset < range_end):
                # Figure out how much data to read
                l = min(constants.SCAN_BLOCKSIZE + overlap, range_end - current_offset)

                data = addr_space.zread(current_offset, l)
            
                for addr in utils.iterfind(data, "\x80\x3D"):
                    full_addr = addr + current_offset 
                    result = self.copy_data_block(full_addr)
                    if result:
                        yield result

                current_offset += min(constants.SCAN_BLOCKSIZE, l)

class Win8x64VolatilityKDBG(obj.ProfileModification):
    """Apply the KDBG finder for x64"""

    before = ['WindowsOverlay', 'WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2, 
                  'memory_model': lambda x: x == "64bit"}
    
    def modification(self, profile):
        profile.object_classes.update({"VolatilityKDBG": VolatilityKDBG})


# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.pstasks as pstasks
import volatility.plugins.mac.common as common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_orphan_threads(pstasks.mac_tasks):
    """Lists threads that don't map back to known modules/processes"""

    def unified_output(self, data):
        common.set_plugin_members(self)

        return TreeGrid([("PID",int),
                        ("Process Name", str),
                        ("Start Address", Address),
                        ("Mapping", str),
                        ("Name", str),
                        ("Status", str),
                        ], self.generator(data))

    def generator(self, data):
        (kstart, kend, kmods) = common.get_kernel_addrs_start_end(self)
        
        for proc in data:
            for thread in proc.threads():
                start = thread.continuation

                if start == 0:
                    continue

                (good, mapping) = common.is_in_kernel_or_module(start, kstart, kend, kmods)

                if not good:
                    mapping = "UNKNOWN"
                    for map in proc.get_proc_maps():
                        if map.links.start <= start <= map.links.end:
                            mapping = map.get_path()
                            if mapping == "":
                                mapping = map.get_special_path()
                       
                            good  = 1 
                            start = map.links.start
 
                if good:
                    status = "OK"
                else:
                    status = "UNKNOWN"

                name = ""
                if thread.uthread:
                    name_buf = self.addr_space.read(thread.uthread.dereference_as("uthread").pth_name, 256)
                    if name_buf:
                        idx = name_buf.find("\x00")
                        if idx != -1:
                            name_buf = name_buf[:idx]
                        
                        name = name_buf

                yield(0, [
                    int(proc.p_pid),
                    str(proc.p_comm),
                    Address(start),
                    str(mapping),
                    str(name),
                    str(status),
                    ])
 
    def render_text(self, outfd, data):
        common.set_plugin_members(self)

        self.table_header(outfd, [("PID","8"),
                                  ("Name", "16"),
                                  ("Start Address", "[addrpad]"),
                                  ("Mapping", "40"),
                                  ("Name", "40"),
                                  ("Status", ""),
                                 ])
 
        (kstart, kend, kmods) = common.get_kernel_addrs_start_end(self)
        
        for proc in data:
            for thread in proc.threads():
                start = thread.continuation
                if start == 0:
                    continue

                (good, mapping) = common.is_in_kernel_or_module(start, kstart, kend, kmods)
                if not good:
                    mapping = "UNKNOWN"
                    for map in proc.get_proc_maps():
                        if map.links.start <= start <= map.links.end:
                            mapping = map.get_path()
                            if mapping == "":
                                mapping = map.get_special_path()
                            good  = 1 
                            start = map.links.start

                status = "UNKNOWN"
                if good:
                    status = "OK"

                name = ""
                if thread.uthread:
                    name_buf = self.addr_space.read(thread.uthread.dereference_as("uthread").pth_name, 256)
                    if name_buf:
                        idx = name_buf.find("\x00")
                        if idx != -1:
                            name_buf = name_buf[:idx]
                        
                        name = name_buf

                self.table_row(outfd, proc.p_pid, proc.p_comm, start, mapping, name, status)



"""
Created on 4 May 2013

@author: mike
"""


class ValidityRoutines(object):
    """Class to hold all validation routines, such as type checking"""

    def type_check(self, value, valid_type):
        """Checks that value is an instance of valid_type, and returns value if it is, or throws a TypeError otherwise

        :param value: The value of which to validate the type
        :type value: object
        :param valid_type: The type against which to validate
        :type valid_type: type
        """
        assert isinstance(value, valid_type), self.__class__.__name__ + " expected " + \
                                              valid_type.__name__ + ", not " + type(value).__name__
        return value

    def class_check(self, klass, valid_class):
        """Checks that class is an instance of valid_class, and returns klass if it is, or throws a TypeError otherwise

        :param klass: Class to validate
        :type klass: class
        :param valid_class: Valid class against which to check class validity
        :type valid_class: class
        """
        assert issubclass(klass, valid_class), self.__class__.__name__ + " expected " + \
                                               valid_class.__name__ + ", not " + klass.__name__

    def confirm(self, assertion, error):
        """Acts like an assertion, but will not be disabled when __debug__ is disabled"""
        if not assertion:
            if error is None:
                error = "An unspecified Assertion was not met in " + self.__class__.__name__
            raise AssertionError(error)



# Volatility
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

import volatility.obj as obj

#--------------------------------------------------------------------------------
# profile modifications
#--------------------------------------------------------------------------------   

class HeapModification(obj.ProfileModification):

    before = ["WindowsObjectClasses"]
    conditions = {'os': lambda x: x == 'windows'}

    def modification(self, profile):
        profile.merge_overlay({
            '_PEB': [ None, {
            'ProcessHeaps': [ None, ['pointer', ['array', lambda x : x.NumberOfHeaps, ['pointer', ['_HEAP']]]]], 
            }],
        })



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
from volatility.renderers import TreeGrid


class mac_print_boot_cmdline(common.AbstractMacCommand):
    """ Prints kernel boot arguments """

    def calculate(self):
        common.set_plugin_members(self)

        pe_state_addr = self.addr_space.profile.get_symbol("_PE_state")
        pe_state = obj.Object("PE_state", offset = pe_state_addr, vm = self.addr_space)
        bootargs = pe_state.bootArgs.dereference_as("boot_args")      
 
        yield bootargs.CommandLine
 
    def unified_output(self, data):
        return TreeGrid([("Command Line", str),
                         ], self.generator(data))

    def generator(self, data):
        for cmdline in data:
            yield(0, [str(cmdline),])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Command Line", "")])
        for cmdline in data:
            self.table_row(outfd, cmdline)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

# based on the plugin described by Cem Gurkok at:
# http://siliconblade.blogspot.co.uk/2013/07/back-to-defense-finding-hooks-in-os-x.html

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import distorm3

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.debug as debug
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_check_syscall_shadow(common.AbstractMacCommand):
    """ Looks for shadow system call tables """

    # https://github.com/siliconblade/volatility/blob/master/mac/check_hooks.py#L216
    def shadowedSyscalls(self, model, distorm_mode, sysents_addr):
        #looks like these syscall functions end with a call to _thread_exception_return
        thread_exc_ret_addr = self.addr_space.profile.get_symbol('_thread_exception_return')

        prev_op = None
        sysent_funcs = ['_unix_syscall_return', '_unix_syscall64', '_unix_syscall']
        for func in sysent_funcs:
            func_addr = self.addr_space.profile.get_symbol(func)
            content = self.addr_space.read(func_addr, 1024)
            for op in distorm3.Decompose(func_addr, content, distorm_mode):
                if not op.valid:
                    break

                if op.mnemonic == "CALL" and op.operands[0].value == thread_exc_ret_addr:
                    break

                if model == "64bit":
                    #callp = &sysent[63] OR &sysent[code] OR callp == sysent
                    if op.mnemonic in ['ADD','CMP'] and op.operands[0].type == 'Register' and op.operands[0].name in ["RSP","RBX","R12","R13","R14","R15"] and 'FLAG_RIP_RELATIVE' in op.flags:
                        #compare actual sysent tbl address to the one in the instruction, calculated per distorm3 INSTRUCTION_GET_RIP_TARGET

                        op_sysent_ptr = obj.Object('Pointer', offset = (op.address + op.operands[1].disp + op.size), vm = self.addr_space)
 
                        if sysents_addr != op_sysent_ptr.v():
                            print "not same: %x | %x" % (sysents_addr, op_sysent_ptr.v())
                            yield (op_sysent_ptr.v(), func, op)
 
                elif model == "32bit":
                    #LEA EAX, [EAX*8+0x82ef20]
                    if op.mnemonic == 'LEA' and op.operands[0].type == 'Register' and op.operands[0].name in ['EDI','EAX'] and distorm3.Registers[op.operands[1].index] == "EAX" and op.operands[1].scale == 8:
                        if op.operands[1].disp != sysents_addr:
                            shadowtbl_addr = op.operands[1].disp
                            yield (shadowtbl_addr, func, op) 
                            break
                    #CMP EAX, 0x82ef20
                    elif op.mnemonic == 'CMP' and op.operands[0].type == 'Register' and op.operands[0].name in ['EDI','EAX'] and prev_op.mnemonic in ['LEA','MOV'] and self.addr_space.is_valid_address(op.operands[1].value) == True:
                        if op.operands[1].value != sysents_addr:
                            shadowtbl_addr = op.operands[1].value
                            yield (shadowtbl_addr, func, op)

                    #CMP DWORD [EBP-0x20], 0x82ef20
                    elif op.mnemonic == 'CMP' and op.operands[0].index != None and distorm3.Registers[op.operands[0].index] == "EBP" and op.operands[0].disp == -32 and op.operands[0].type == "Immediate":
                        if op.operands[1].value != sysents_addr:
                            shadowtbl_addr = op.operands[1].value
                            yield (shadowtbl_addr, func, op)
 
                prev_op = op

    def calculate(self):
        common.set_plugin_members(self)

        model = self.addr_space.profile.metadata.get('memory_model', 0)

        if model == '32bit':
            distorm_mode = distorm3.Decode32Bits
        else:
            distorm_mode = distorm3.Decode64Bits
        
        for (shadowtbl_addr, func, op) in self.shadowedSyscalls(model, distorm_mode, self.addr_space.profile.get_symbol("_sysent")):
            yield (shadowtbl_addr, func, op)

    def unified_output(self, data):
        return TreeGrid([("Hooked Function", str),
                          ("Hook Address", Address),
                          ("Instruction", str),
                          ], self.generator(data))

    def generator(self, data):
        for (shadowtbl_addr, func, op) in data:
            yield(0, [
                str(func),
                Address(shadowtbl_addr),
                str(op),
                ])

    def render_text(self, outfd, data):
        self.table_header(outfd, 
                          [("Hooked Function", "30"),
                          ("Hook Address", "[addrpad]"),
                          ("Instruction", "")])

        for (shadowtbl_addr, func, op) in data:
            self.table_row(outfd, func, shadowtbl_addr, op)



# Volatility
# Copyright (C) 2008-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
This module implements the fast connection scanning

@author:       AAron Walters and Brendan Dolan-Gavitt
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net,bdolangavitt@wesleyan.edu
@organization: Volatility Foundation
"""

#pylint: disable-msg=C0111

import volatility.poolscan as poolscan
import volatility.plugins.common as common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class PoolScanConn(poolscan.PoolScanner):
    """Pool scanner for tcp connections"""

    def __init__(self, address_space):
        poolscan.PoolScanner.__init__(self, address_space)

        self.struct_name = "_TCPT_OBJECT"
        self.pooltag = "TCPT"

        self.checks = [ ('CheckPoolSize', dict(condition = lambda x: x >= 0x198)),
                   ('CheckPoolType', dict(non_paged = True, free = True)),
                   ('CheckPoolIndex', dict(value = lambda x : x < 5)),
                   ]

class ConnScan(common.AbstractScanCommand):
    """Pool scanner for tcp connections"""

    scanners = [PoolScanConn]

    meta_info = dict(
        author = 'Brendan Dolan-Gavitt',
        copyright = 'Copyright (c) 2007,2008 Brendan Dolan-Gavitt',
        contact = 'bdolangavitt@wesleyan.edu',
        license = 'GNU General Public License 2.0',
        url = 'http://moyix.blogspot.com/',
        os = 'WIN_32_XP_SP2',
        version = '1.0',
        )

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('major', 0) == 5)

    def render_text(self, outfd, data):
        self.table_header(outfd,
                          [(self.offset_column(), "[addrpad]"),
                           ("Local Address", "25"),
                           ("Remote Address", "25"),
                           ("Pid", "")
                           ])

        for tcp_obj in data:
            local = "{0}:{1}".format(tcp_obj.LocalIpAddress, tcp_obj.LocalPort)
            remote = "{0}:{1}".format(tcp_obj.RemoteIpAddress, tcp_obj.RemotePort)
            self.table_row(outfd,
                            tcp_obj.obj_offset,
                            local, remote,
                            tcp_obj.Pid)

    def unified_output(self, data):
        return TreeGrid([("Offset(P)", Address),
                       ("LocalAddress", str),
                       ("RemoteAddress", str),
                       ("PID", int)],
                        self.generator(data))

    def generator(self, data):
        for conn in data:
            local = "{0}:{1}".format(conn.LocalIpAddress, conn.LocalPort)
            remote = "{0}:{1}".format(conn.RemoteIpAddress, conn.RemotePort)
            yield (0, [Address(conn.obj_offset), str(local), str(remote), int(conn.Pid)])



# Volatility
# Copyright (C) 2008-2013 Volatility Foundation
# Copyright (C) 2011 Jamie Levy (Gleeda) <jamie@memoryanalysis.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Jamie Levy (Gleeda)
@license:      GNU General Public License 2.0
@contact:      jamie@memoryanalysis.net
@organization: Volatility Foundation
"""

import volatility.utils as utils
import volatility.win32.hive as hivemod
import volatility.win32.rawreg as rawreg
import volatility.win32.hashdump as hashdump
import volatility.utils as utils
import volatility.plugins.registry.hivelist as hl
from heapq import nlargest


class RegistryApi(object):
    """A wrapper several highly used Registry functions"""

    def __init__(self, config):
        self._config = config
        self.addr_space = utils.load_as(self._config)
        self.all_offsets = {}
        self.current_offsets = {}
        self.populate_offsets()


    def print_offsets(self):
        '''
        this is just in case we want to check our offsets and which hive(s) was/were chosen
        '''
        for item in self.all_offsets:
            print "0x{0:x}".format(item), self.all_offsets[item]
        for item in self.current_offsets:
            print 'current', "0x{0:x}".format(item), self.current_offsets[item]

    def populate_offsets(self):
        '''
        get all hive offsets so we don't have to scan again...
        '''
        hive_offsets = []
        hiveroot = hl.HiveList(self._config).calculate()

        for hive in hiveroot:
            if hive.is_valid() and hive.obj_offset not in hive_offsets:
                hive_offsets.append(hive.obj_offset)
                self.all_offsets[hive.obj_offset] = hive.get_name()

    def reg_get_currentcontrolset(self, fullname = True):
        '''
        get the CurrentControlSet
            If fullname is not specified, we only get the number like "1" or "2" etc
            The default is ControlSet00{#} so we can append it to the desired key path
            We return None if it fails, so you need to verify before using.
        '''
        for offset in self.all_offsets:
            name = self.all_offsets[offset] + " "
            if name.lower().find("\\system ") != -1:
                sysaddr = hivemod.HiveAddressSpace(self.addr_space, self._config, offset)
                if fullname:
                    return "ControlSet00{0}".format(hashdump.find_control_set(sysaddr))
                else:
                    return hashdump.find_control_set(sysaddr)
        return None

    def set_current(self, hive_name = None, user = None):
        '''
        if we find a hive that fits the given criteria, save its offset 
        so we don't have to scan again.  this can be reset using reset_current
        if context changes
        '''
        for item in self.all_offsets:
            name = self.all_offsets[item] + " "
            if user == None and hive_name == None:
                #no particular preference: all hives
                self.current_offsets[item] = name
            elif user != None and name.lower().find('\\' + user.lower() + '\\') != -1 and name.lower().find("\\" + "ntuser.dat ") != -1:
                #user's NTUSER.DAT hive
                self.current_offsets[item] = name
            elif hive_name != None and hive_name.lower() == 'hklm' \
                and (name.lower().find("\\security ") != -1 or name.lower().find("\\system ") != -1 \
                or name.lower().find("\\software ") != -1 or name.lower().find("\\sam ") != -1):
                #any HKLM hive 
                self.current_offsets[item] = name
            elif hive_name != None and name.lower().find("\\" + hive_name.lower() + " ") != -1 and user == None:
                #a particular hive indicated by hive_name
                if hive_name.lower() == "system" and name.lower().find("\\syscache.hve ") == -1:
                    self.current_offsets[item] = name
                elif hive_name.lower() != "system":
                    self.current_offsets[item] = name

    def reset_current(self):
        '''
        this is in case we switch to a different hive/user/context
        '''
        self.current_offsets = {}

    def reg_get_key(self, hive_name, key, user = None, given_root = None):
        '''
        Returns a key from a requested hive; assumes this is from a single hive
        if more than one hive is specified, the hive/key found is returned
        '''
        if self.all_offsets == {}:
            self.populate_offsets()
        if self.current_offsets == {}:
            self.set_current(hive_name, user)
        if key:
            for offset in self.current_offsets:
                if given_root == None:
                    h = hivemod.HiveAddressSpace(self.addr_space, self._config, offset)
                    root = rawreg.get_root(h)
                else:
                    root = given_root
                if root != None:
                    k = rawreg.open_key(root, key.split('\\'))
                    if k:
                        return k
        return None

    def reg_get_key_path(self, key):
        ''' 
        Takes in a key object and traverses back through its family to build the path
        '''
        path = key.Name
        while key.Parent and key.Parent & 0xffffffff > 0x20:
            key = key.Parent.dereference()
            if utils.remove_unprintable(str(key.Name)) != "": 
                path = "{0}\\{1}".format(key.Name, path)
        return path

    def reg_yield_key(self, hive_name, key, user = None, given_root = None):
        ''' 
        Use this function if you are collecting keys from more than one hive
        '''
        if self.all_offsets == {}:
            self.populate_offsets()
        if self.current_offsets == {}:
            self.set_current(hive_name, user)
        if key:
            for offset in self.current_offsets:
                name = self.current_offsets[offset]
                if given_root == None:
                    h = hivemod.HiveAddressSpace(self.addr_space, self._config, offset)
                    root = rawreg.get_root(h)
                else:
                    root = given_root
                if root != None:
                    k = rawreg.open_key(root, key.split('\\'))
                    if k:
                        yield k, name

    def reg_enum_key(self, hive_name, key, user = None):
        '''
        This function enumerates the requested key
        '''
        k = self.reg_get_key(hive_name, key, user)
        if k:
            for s in rawreg.subkeys(k):
                if s.Name:
                    item = key + '\\' + s.Name
                    yield item

    def reg_get_all_subkeys(self, hive_name, key, user = None, given_root = None):
        '''
        This function enumerates the subkeys of the requested key
        '''
        if key or given_root:
            k = given_root if given_root != None else self.reg_get_key(hive_name, key)
            if k:
                for s in rawreg.subkeys(k):
                    if s.Name:
                        yield s

    def reg_yield_values(self, hive_name, key, thetype = None, given_root = None, raw = False):
        '''
        This function yields all values for a  requested registry key
        '''
        if key or given_root:
            h = given_root if given_root != None else self.reg_get_key(hive_name, key)
            if h != None:
                for v in rawreg.values(h):
                    tp, dat = rawreg.value_data(v)
                    if thetype == None or tp == thetype:
                        if raw:
                            yield v, dat
                        else:
                            yield v.Name, dat 

    def reg_get_value(self, hive_name, key, value, strcmp = None, given_root = None):
        '''
        This function returns the requested value of a registry key
        '''
        if value != None:
            if given_root == None and key != None:
                given_root = self.reg_get_key(hive_name, key)
            if given_root != None:
                for v in rawreg.values(given_root):
                    if value == v.Name:
                        tp, dat = rawreg.value_data(v)
                        if tp == 'REG_BINARY' or strcmp == None:
                            # We want raw data
                            return dat
                        else:
                            # This is a string comparison
                            dat = str(dat)
                            dat = dat.strip()
                            dat = ''.join([x for x in dat if ord(x) != 0])  #get rid of funky nulls for string comparison
                            if strcmp == dat:
                                return dat
        return None

    def reg_get_all_keys(self, hive_name, user = None, start = None, end = None, reg = False, rawtime = False):
        '''
        This function enumerates all keys in specified hives and 
        collects lastwrite times.
        '''
        keys = []
        if self.all_offsets == {}:
            self.populate_offsets()
        if self.current_offsets == {}:
            self.set_current(hive_name, user)

        # Collect the root keys 
        for offset in self.current_offsets:
            reg_name = self.current_offsets[offset]
            h = hivemod.HiveAddressSpace(self.addr_space, self._config, offset)
            root = rawreg.get_root(h)
            if not root:
                pass
            else:
                time = "{0}".format(root.LastWriteTime) if not rawtime else root.LastWriteTime
                if reg:
                    if start and end and str(time) >= start and str(time) <= end:
                        yield (time, reg_name, root.Name)
                    elif start == None and end == None:
                        yield (time, reg_name, root.Name)
                else:
                    if start and end and str(time) >= start and str(time) <= end:
                        yield (time, root.Name)
                    elif start == None and end == None:
                        yield (time, root.Name)
                for s in rawreg.subkeys(root):
                    if reg:
                        keys.append([s, reg_name, root.Name + "\\" + s.Name])
                    else:
                        keys.append([s, root.Name + "\\" + s.Name])

        # Get subkeys
        if reg:
            for k, reg_name, name in keys:
                time = "{0}".format(k.LastWriteTime) if not rawtime else k.LastWriteTime
                if start and end and str(time) >= start and str(time) <= end:
                    yield (time, reg_name, name)
                elif start == None and end == None:
                    yield (time, reg_name, name)
                for s in rawreg.subkeys(k):
                    if name and s.Name:
                        item = name + '\\' + s.Name
                        keys.append([s, reg_name, item])
        else:
            for k, name in keys:
                time = "{0}".format(k.LastWriteTime) if not rawtime else k.LastWriteTime
                if start and end and str(time) >= start and str(time) <= end:
                    yield (time, name)
                elif start == None and end == None:
                    yield (time, name)

                for s in rawreg.subkeys(k):
                    if name and s.Name:
                        item = name + '\\' + s.Name
                        keys.append([s, item])

    def reg_get_last_modified(self, hive_name, count = 1, user = None, start = None, end = None, reg = False):
        '''
        Wrapper function using reg_get_all_keys. These functions can take a WHILE since all 
        subkeys have to be collected before you can compare lastwrite times.
        '''
        data = nlargest(count, self.reg_get_all_keys(hive_name, user, start, end, reg))
        if reg:
            for t, regname, name in data:
                yield (t, regname, name)
        else:
            for t, name in data: 
                yield (t, name)





# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import os

import volatility.obj   as obj
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.find_file as linux_find_file

class linux_recover_filesystem(linux_common.AbstractLinuxCommand):
    """Recovers the entire cached file system from memory"""

    def __init__(self, config, *args, **kwargs):
        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)
        self._config.add_option('DUMP-DIR', short_option = 'D', default = None, help = 'Output directory', action = 'store', type = 'str')

    def _fix_metadata(self, file_path, file_dentry):
        inode = file_dentry.d_inode
        
        if inode and inode.is_valid():
            ents = file_path.split("/")
            out_path = os.path.join(self._config.DUMP_DIR, *ents)

            os.chmod(out_path, inode.i_mode & 00777)
            os.chown(out_path, inode.i_uid, inode.i_gid)
            os.utime(out_path, (inode.i_atime.tv_sec, inode.i_mtime.tv_sec))

    def _write_file(self, ff, file_path, file_dentry):
        inode = file_dentry.d_inode
        
        if inode and inode.is_valid() and not inode.is_dir():
            ents = file_path.split("/")
            out_path = os.path.join(self._config.DUMP_DIR, *ents)

            try:
                fd = open(out_path, "wb")
            except IOError, e:
                debug.warning("Unable to process file: %s : %s" % (out_path, str(e)))
                return
                
            for page in ff.get_file_contents(inode):
                fd.write(page)  
            
            fd.close()
            
    def _make_path(self, file_path, file_dentry):
        inode = file_dentry.d_inode
        
        if inode.is_dir():
            ents = file_path.split("/")
        else:
            ents = file_path.split("/")[:-1]

        out_path = os.path.join(self._config.DUMP_DIR, *ents)

        try:
            os.makedirs(out_path)
        except OSError:
            pass

    def calculate(self):
        linux_common.set_plugin_members(self)
        
        num_files = 0

        if (not self._config.DUMP_DIR or not os.path.isdir(self._config.DUMP_DIR)):
            debug.error("Please specify an existing output dir (--dump-dir)")

        ff = linux_find_file.linux_find_file(self._config)

        for (_, _, file_path, file_dentry) in ff.walk_sbs():
            self._make_path(file_path, file_dentry)
            self._write_file(ff, file_path, file_dentry)
            self._fix_metadata(file_path, file_dentry)

            num_files = num_files + 1

        yield num_files

    def render_text(self, outfd, data):
        for (num_files) in data: 
            outfd.write("Recovered %d files\n" % num_files)




# Volatility
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

'''
This helper script generates (kernel version, version address pairs)
to help generate the list used by mac_get_profiles

Run it from the Mac directory of the Volatility profiles repo
'''

import os, sys, re
import zipfile

def parse_dsymutil(data, module):
    """Parse the symbol file."""
    sys_map = {}
    sys_map[module] = {}

    want_lower = ["_IdlePML4"]        

    type_map = {}
    type_map[module] = {}

    # get the system map
    for line in data.splitlines():
        ents = line.split()

        match = re.search("\[.*?\(([^\)]+)\)\s+[0-9A-Fa-z]+\s+\d+\s+([0-9A-Fa-f]+)\s'(\w+)'", line)

        if match:
            (sym_type, addr, name) = match.groups()
            sym_type = sym_type.strip()
    
            addr = int(addr, 16)

            if addr == 0 or name == "":
                continue

            if not name in sys_map[module]:
                sys_map[module][name] = [(addr, sym_type)]
                
            # every symbol is in the symbol table twice
            # except for the entries in 'want_lower', we need the higher address for all 
            oldaddr = sys_map[module][name][0][0]
            if addr < oldaddr and name in want_lower:
                sys_map[module][name] = [(addr, sym_type)]
        
            if not addr in type_map[module]:
                type_map[module][addr] = (name, [sym_type])

            type_map[module][addr][1].append(sym_type)

    return sys_map["kernel"]

print "profiles = ["

for path in set("."):
    for path, _, files in os.walk(path):
        for fn in files:
            if zipfile.is_zipfile(os.path.join(path, fn)):
                profpkg = zipfile.ZipFile(os.path.join(path, fn))

                for f in profpkg.filelist:
                    if 'symbol.dsymutil' in f.filename.lower():
                        data = parse_dsymutil(profpkg.read(f.filename), "kernel")
           
                        if "_lowGlo" in data:
                            lg = data["_lowGlo"][0][0]
                        else:
                            lg = "0"

                        if "_BootPML4" in data:
                            aslr = 1
                        else:
                            aslr = 0

                        name = fn.replace(".zip", "")
                        name = 'Mac' + name.replace('.', '_')

                        if name.find("Intel") == -1:
                            name = name + "x64"
                        else:
                            name = name + "x86"

                        print "[\"%s\", %s, %s, %d]," % (name, data["_version"][0][0], lg, aslr)

print "]"




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj   as obj
import volatility.utils as utils
import volatility.debug as debug
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address
import volatility.plugins.mac.common as common

class mac_list_kauth_scopes(common.AbstractMacCommand):
    """ Lists Kauth Scopes and their status """

    def calculate(self):
        common.set_plugin_members(self)

        scopes_addr = self.addr_space.profile.get_symbol("_kauth_scopes")
        scopes_ptr = obj.Object("Pointer", offset = scopes_addr, vm = self.addr_space)
        scope = scopes_ptr.dereference_as("kauth_scope")
        
        while scope.is_valid():
            yield scope
            scope = scope.ks_link.tqe_next.dereference()

    def unified_output(self, data):
        common.set_plugin_members(self)
        
        return TreeGrid([("Offset", Address),
                          ("Name", str),
                          ("IData", Address),
                          ("Listeners", int),
                          ("Callback Addr", Address),
                          ("Callback Mod", str),
                          ("Callback Sym", str),
                          ], self.generator(data))

    def generator(self, data):
        kaddr_info = common.get_handler_name_addrs(self)

        for scope in data:
            cb = scope.ks_callback.v()
            (module, handler_sym) = common.get_handler_name(kaddr_info, cb)

            yield(0, [
                Address(scope.v()),
                str(scope.ks_identifier),
                Address(scope.ks_idata),
                int(len([l for l in scope.listeners()])),
                Address(cb),
                str(module),
                str(handler_sym),
                ])

    def render_text(self, outfd, data):
        common.set_plugin_members(self)
        
        self.table_header(outfd, [("Offset", "[addrpad]"),
                          ("Name", "24"),
                          ("IData", "[addrpad]"),
                          ("Listeners", "5"),
                          ("Callback Addr", "[addrpad]"),
                          ("Callback Mod", "24"),
                          ("Callback Sym", ""),])

        kaddr_info = common.get_handler_name_addrs(self)
        
        for scope in data:
            cb = scope.ks_callback.v()
            (module, handler_sym) = common.get_handler_name(kaddr_info, cb) 
            
            self.table_row(outfd, scope.v(),
                                  scope.ks_identifier,
                                  scope.ks_idata,
                                  len([l for l in scope.listeners()]), 
                                  cb, module, handler_sym)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2014 CrowdStrike, Inc.
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Georg Wicherski
@license:      GNU General Public License 2.0
@contact:      georg@crowdstrike.com
@organization: CrowdStrike, Inc.
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as linux_pslist

class linux_plthook(linux_pslist.linux_pslist):
    """Scan ELF binaries' PLT for hooks to non-NEEDED images"""

    def __init__(self, config, *args, **kwargs):
        linux_pslist.linux_pslist.__init__(self, config, *args, **kwargs)
        self._config.add_option( \
                'ALL', short_option = 'a', default = False,
                help = 'Display all PLT slots (incl. not hooked)', action = 'store_true')
        self._config.add_option( \
                'IGNORE', default = [ ],
                help = 'Ignore mappings backed by this path, ' \
                        +' useful for bad -l compiles (i.e. apache2 modules)',
                        action = 'append')

    def render_text(self, outfd, data):
        linux_common.set_plugin_members(self)
        
        self.table_header(outfd, [("Task", "10"), 
                                  ("ELF Start", "[addrpad]"), 
                                  ("ELF Name", "24"),
                                  ("Symbol", "24"),
                                  ("Resolved Address", "[addrpad]"),
                                  ("H", "1"),
                                  ("Target Info", "")])
            
        ignore = frozenset(self._config.IGNORE)

        for task in data:
            for soname, elf, elf_start, elf_end, addr, symbol_name, hookdesc, hooked in task.plt_hook_info():
                if not hooked and not self._config.ALL:
                    continue

                if hookdesc in ignore:
                    continue

                if hookdesc == '[RTLD_LAZY]' and not self._config.ALL:
                    continue

                self.table_row(outfd, task.pid, elf_start, soname if soname else '[main]', \
                    symbol_name, addr, '!' if hooked else ' ', hookdesc)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.utils as utils
import volatility.plugins.mac.pstasks as pstasks 
from volatility.renderers import TreeGrid

class mac_contacts(pstasks.mac_tasks):
    """Gets contact names from Contacts.app"""

    def calculate(self):
        common.set_plugin_members(self)

        procs = pstasks.mac_tasks.calculate(self)

        for proc in procs:
            space = proc.get_process_address_space()
            for map in proc.get_proc_maps():

                # only read/write without filebacks 
                if not (map.get_perms() == "rw-" and not map.get_path()):
                    continue

                # check the header for sqlite3 signature 
                header = space.zread(map.links.start, 32)
                if "SQLite format" not in header:
                    continue

                # get the whole sqlite3 data now 
                data = space.zread(map.links.start, 
                                  map.links.end - map.links.start)
                
                for offset in utils.iterfind(data, ":ABPerson"):
                    person = obj.Object("String", 
                                        offset = map.links.start + offset, 
                                        vm = space, encoding = "utf8", 
                                        length = 256)
                    yield proc, person
    
    def unified_output(self, data):
        return TreeGrid([("Contact", str),
                         ],
                         self.generator(data))
                         
    def generator(self, data):
        for (proc, person) in data:
            # strip the header from the string 
            person = str(person)[len(":ABPerson"):]

            # take a maximum of eight parts  
            items = " ".join(person.split(" ")[:8])
            
            yield(0, [str(items),])            

    def render_text(self, outfd, data):
        for (proc, person) in data:
            # strip the header from the string 
            person = str(person)[len(":ABPerson"):]

            # take a maximum of eight parts  
            items = " ".join(person.split(" ")[:8])
            
            outfd.write("{0}\n".format(items))



# Volatility
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""
@author: Mariano `emdel` Graziano, Edwin Smulders
@license: GNU General Public License 2.0 or later
@contact: graziano@eurecom.fr, mail@edwinsmulders.eu
"""


import volatility.plugins.linux.common as common
import volatility.plugins.linux.pslist as linux_pslist
import collections
import struct
import volatility.debug as debug

offsets = {}
# x86 offsets - It works on my Linux machine.
offsets['32bit'] = [
'ebx', 
'ecx',
'edx', 
'esi', 
'edi', 
'ebp', 
'eax', 
'ds', 
'es', 
'fs', 
'gs', 
'orig_eax', 
'eip', 
'cs', 
'eflags', 
'esp', 
'ss'
]

# x64 offsets
offsets['64bit'] = [
'r15', 
'r14', 
'r13', 
'r12',
'rbp', 
'rbx',
'r11', 
'r10', 
'r9', 
'r8',
'rax', 
'rcx', 
'rdx',
'rsi', 
'rdi',
'unknown', # I'm not sure what this field is
'rip',
'cs', 
'eflags', 
'rsp', 
'ss'
]

reg_size = {}
reg_size['32bit'] = 0x4
reg_size['64bit'] = 0x8

fmt = {}
fmt['32bit'] = '<I'
fmt['64bit'] = '<Q'


class linux_info_regs(linux_pslist.linux_pslist):
    '''It's like 'info registers' in GDB. It prints out all the
processor registers involved during the context switch.'''
    def __init__(self, config, *args, **kwargs):
        linux_pslist.linux_pslist.__init__(self, config, *args, **kwargs) 
        
        self.bits     = 0
        self.reg_size = 0
        self.offsets  = []
        self.fmt      = ""

    def calculate(self):
        common.set_plugin_members(self)

        if self.profile.metadata['arch'] not in ["x64", "x86"]:
            debug.error("This plugin is only supported on Intel-based memory captures") 

        self.bits = self.profile.metadata.get('memory_model', '32bit')
        self.reg_size = reg_size[self.bits]
        self.offsets = offsets[self.bits]
        self.fmt = fmt[self.bits]

        for proc in linux_pslist.linux_pslist(self._config).calculate():
            name = proc.get_commandline()
            thread_registers = []
            for thread_task in proc.threads():
                thread_name = thread_task.comm
                regs = self.parse_kernel_stack(thread_task)
                thread_registers.append((thread_name,regs))
            yield proc, name, thread_registers

    def render_text(self, outfd, data):

        #outfd.write("[-- Info Registers:\n")

        for task, name, thread_regs in data:
            outfd.write("Process Name: {} - PID: {}\n".format(name, str(task.pid)))
            outfd.write("Registers (per thread):\n")
            fmt = str(2*self.reg_size)
            for thread_name, regs in thread_regs:
                outfd.write("  Thread Name: {}\n".format(thread_name))
                if regs != None:
                    for reg, value in regs.items():

                        outfd.write(("    {:8s}: {:0" + fmt + "x}\n").format(reg, value))


    def parse_kernel_stack(self, task):
        result = collections.OrderedDict()
        if 1 or task.mm:
            sp0 = task.thread.sp0
            #proc_as = task.get_process_address_space()
            addr = sp0

            for reg in self.offsets[::-1]: # reverse list, because we read up in the stack
                #debug.info("Reading {:016x}".format(addr))
                addr -= self.reg_size
                val_raw = self.addr_space.read(addr, self.reg_size)
                val = struct.unpack(self.fmt, val_raw)[0]
                result[reg] = val
            return result
        return None



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common

class mac_version(common.AbstractMacCommand):
    """ Prints the Mac version """

    def calculate(self):
        common.set_plugin_members(self)
        yield obj.Object("String", offset = self.addr_space.profile.get_symbol("_version"), vm = self.addr_space, length = 256)
 
    def render_text(self, outfd, data):
        for version in data:
            outfd.write("{0}\n".format(version))



# Volatility
# Copyright (C) 2007-2014 Volatility Foundation
#
# Authors: 
# phil@teuwen.org (Philippe Teuwen)
# espen@mrfjo.org (Espen Fjellvaer Olsen)
# justincapella@gmail.com (Justin Capella)
# michael.ligh@mnin.org (Michael Ligh)
# 
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#
# References:
# VirtualBox core format: 
#     http://www.virtualbox.org/manual/ch12.html#guestcoreformat
#     http://www.virtualbox.org/svn/vbox/trunk/include/VBox/vmm/dbgfcorefmt.h
#     http://www.virtualbox.org/svn/vbox/trunk/src/VBox/VMM/VMMR3/DBGFCoreWrite.cpp

import volatility.obj as obj
import volatility.addrspace as addrspace

#pylint: disable-msg=C0111

class OSXPmemELF(addrspace.AbstractRunBasedMemory):
    """ This AS supports VirtualBox ELF64 coredump format """

    order = 90

    def __init__(self, base, config, **kwargs):
        ## We must have an AS below us
        self.as_assert(base, "No base Address Space")
        addrspace.AbstractRunBasedMemory.__init__(self, base, config, **kwargs)

        ## Quick test (before instantiating an object) 
        ## for ELF64, little-endian - ELFCLASS64 and ELFDATA2LSB
        ## for ELF32, little-endian - ELFCLASS32 and ELFDATA2LSB
        self.as_assert(base.read(0, 6) in ['\x7fELF\x02\x01', '\x7fELF\x01\x01'], "ELF Header signature invalid")

        ## Base AS should be a file AS
        elf = obj.Object("elf_hdr", offset = 0, vm = base)

        ## The PT_NOTE core descriptor structure 
        self.header = None

        for phdr in elf.program_headers():

            # Only keep load segments with valid file sizes
            if (str(phdr.p_type) != 'PT_LOAD' or
                    phdr.p_filesz == 0 or
                    phdr.p_filesz != phdr.p_memsz):
                continue

            self.runs.append((int(phdr.p_paddr),
                              int(phdr.p_offset),
                              int(phdr.p_memsz)))

        self.as_assert(len(self.runs) > 0, "No PT_LOAD segments found")





# Volatility
#
# Zeus support:
# Michael Hale Ligh <michael.ligh@mnin.org>
#
# Citadel support:
# Santiago Vicente <smvicente@invisson.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import struct, hashlib
import volatility.utils as utils
import volatility.obj as obj
import volatility.commands as commands
import volatility.debug as debug
import volatility.win32.tasks as tasks
import volatility.plugins.malware.impscan as impscan
import volatility.plugins.taskmods as taskmods
import volatility.plugins.procdump as procdump
import volatility.addrspace as addrspace
import volatility.plugins.vadinfo as vadinfo
import volatility.exceptions as exceptions

try:
    import yara
    has_yara = True
except ImportError:
    has_yara = False

RC4_KEYSIZE = 0x102

#--------------------------------------------------------------------------------
# Profile Modifications 
#--------------------------------------------------------------------------------

class ZeusVTypes(obj.ProfileModification):

    conditions = {'os': lambda x: x == 'windows', 
                  'memory_model': lambda x: x == "32bit"}
    
    def modification(self, profile):
        profile.vtypes.update({
            '_ZEUS2_CONFIG' : [ 0x1E6, {
                'struct_size' :   [ 0x0, ['unsigned int']], 
                'guid' :   [ 0x4, ['array', 0x30, ['unsigned short']]], 
                'guid2' : [ 0x7C, ['array', 0x10, ['unsigned char']]], 
                'rc4key' : [ 0x8C, ['array', 0x100, ['unsigned char']]], 
                'exefile' : [ 0x18E, ['String', dict(length = 0x14)]], 
                'datfile' : [ 0x1A2, ['String', dict(length = 0x14)]], 
                'keyname' : [ 0x1B6, ['String', dict(length = 0xA)]], 
                'value1' : [ 0x1C0, ['String', dict(length = 0xA)]],  
                'value2' : [ 0x1CA, ['String', dict(length = 0xA)]], 
                'value3' : [ 0x1D4, ['String', dict(length = 0xA)]], 
                'guid_xor_key' : [ 0x1DE, ['unsigned int']], 
                'xorkey' : [ 0x1E2, ['unsigned int']], 
            }], 
            '_CITADEL1345_CONFIG' : [ 0x11C, {
                'struct_size' :   [ 0x0, ['unsigned int']], 
                'guid' :   [ 0x4, ['array', 0x30, ['unsigned short']]], 
                'guid2' : [ 0x7C, ['array', 0x10, ['unsigned char']]], 
                'exefile' : [ 0x9C, ['String', dict(length = 0x14)]], 
                'datfile' : [ 0xB0, ['String', dict(length = 0x14)]], 
                'keyname' : [ 0xEC, ['String', dict(length = 0xA)]], 
                'value1' : [ 0xF6, ['String', dict(length = 0xA)]],  
                'value2' : [ 0x100, ['String', dict(length = 0xA)]], 
                'value3' : [ 0x10A, ['String', dict(length = 0xA)]], 
                'guid_xor_key' : [ 0x114, ['unsigned int']], 
                'xorkey' : [ 0x118, ['unsigned int']], 
                }],
            })

#--------------------------------------------------------------------------------
# Scanner for Zeus > 1.20 and < 2.0  
#--------------------------------------------------------------------------------

class ZeusScan1(taskmods.DllList):
    """Locate and Decrypt Zeus > 1.20 and < 2.0 Configs"""

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('memory_model', '32bit') == '32bit')

    def _zeus_filter(self, vad):
        """
        This is a callback that's executed by get_vads() 
        when searching for zeus injections. 

        @param vad: an MMVAD object.

        @returns: True if the MMVAD looks like it might
        contain a zeus image. 

        We want the memory to be executable, but right now we 
        can only get the original protection not the current 
        protection...and the original protection can be 
        anything. This version of zeus happens to use 
        PAGE_NOACCESS so that's what we'll look for instead.
        """

        prot = vad.u.VadFlags.Protection.v()
        prot = vadinfo.PROTECT_FLAGS.get(prot, "")

        return (vad.u.VadFlags.PrivateMemory == 0 and 
                    prot == "PAGE_NO_ACCESS" and 
                    vad.Tag == "VadS")

    def calculate(self):
        
        addr_space = utils.load_as(self._config)

        if not self.is_valid_profile(addr_space.profile):
            debug.error("This command does not support the selected profile.")

        for task in self.filter_tasks(tasks.pslist(addr_space)):
            task_space = task.get_process_address_space()

            # We must have a process AS
            if not task_space:
                continue 

            winsock = None

            # Locate the winsock DLL
            for mod in task.get_load_modules():
                if str(mod.BaseDllName or '').lower() == "ws2_32.dll":
                    winsock = mod
                    break
            
            if not winsock:
                continue 

            # Resolve the closesocket API 
            closesocket = winsock.getprocaddress("closesocket")

            if not closesocket:
                continue 

            for vad, process_space in task.get_vads(
                                    vad_filter = self._zeus_filter,
                                    ):
                                    
                if obj.Object("_IMAGE_DOS_HEADER", offset = vad.Start, 
                        vm = process_space).e_magic != 0x5A4D:
                    continue
                    
                data = process_space.zread(vad.Start, vad.Length)
            
                scanner = impscan.ImpScan(self._config).call_scan
                calls = list(scanner(task_space, vad.Start, data))

                for (_, iat_loc, call_dest) in calls:
                    if call_dest != closesocket:
                        continue 

                    # Read the DWORD directly after closesocket 
                    struct_base = obj.Object('Pointer', 
                        offset = iat_loc + 4, vm = task_space)

                    # To be valid, it must point within the vad segment 
                    if (struct_base < vad.Start or 
                            struct_base > (vad.Start + vad.End)):
                        continue 

                    # Grab the key data
                    key = task_space.read(struct_base + 0x2a, RC4_KEYSIZE)

                    # Greg's sanity check
                    if len(key) != RC4_KEYSIZE or key[-2:] != "\x00\x00":
                        continue

                    yield task, struct_base, key

    def render_text(self, outfd, data):

        for task, struct_base, key in data:
            hex = "\n".join(["{0:#010x}  {1:<48}  {2}".format(
                            struct_base + 0x2a + o, 
                            h, ''.join(c)) for o, h, c in utils.Hexdump(key)
                            ])
            outfd.write("Process: {0} {1}\n".format(
                task.UniqueProcessId, task.ImageFileName))
            outfd.write(hex)
            outfd.write("\n")

#--------------------------------------------------------------------------------
# Scanner for Zeus >= 2.0 
#--------------------------------------------------------------------------------

class ZeusScan2(procdump.ProcDump):
    """Locate and Decrypt Zeus >= 2.0 Configs"""

    signatures = {
    'namespace1':'rule z1 {strings: $a = {56 BA ?? ?? 00 00 52 68 ?? ?? ?? ?? 50 E8 ?? ?? ?? ?? 8B 35 ?? ?? ?? ?? 8B 0D ?? ?? ?? ??} condition: $a}',
    'namespace5':'rule z5 {strings: $a = {56 BA ?? ?? 00 00 52 68 ?? ?? ?? ?? 50 E8 ?? ?? ?? ?? 8B 0D ?? ?? ?? ?? 03 0D ?? ?? ?? ??} condition: $a}',
    'namespace2':'rule z2 {strings: $a = {55 8B EC 51 A1 ?? ?? ?? ?? 8B 0D ?? ?? ?? ?? 56 8D 34 01 A1 ?? ?? ?? ?? 8B 0D ?? ?? ?? ??} condition: $a}',
    'namespace3':'rule z3 {strings: $a = {68 02 01 00 00 8D 84 24 ?? ?? ?? ?? 50 8D 44 24 ?? 50 E8 ?? ?? ?? ?? B8 E6 01 00 00 50 68 ?? ?? ?? ??} condition: $a}',
    'namespace4':'rule z4 {strings: $a = {68 02 01 00 00 8D 85 ?? ?? ?? ?? 50 8D 85 ?? ?? ?? ?? 50 E8 ?? ?? ?? ?? B8 E6 01 00 00 50 68 ?? ?? ?? ??} condition: $a}'
    }

    magic_struct = '_ZEUS2_CONFIG'

    params = dict(
            # This contains the C2 URL, RC4 key for decoding 
            # local.ds and the magic buffer
            decoded_config = None,
            # This contains the hardware lock info, the user.ds 
            # RC4 key, and XOR key
            encoded_magic = None,
            # The decoded version of the magic structure
            decoded_magic = None, 
            # The key for decoding the configuration
            config_key = None, 
            # The login key (citadel only)
            login_key = None, 
            # The AES key (citadel only)
            aes_key = None, 
            )

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('memory_model', '32bit') == '32bit')

    def rc4(self, key, encoded):
        """Perform a basic RC4 operation"""
        # Turn the buffers into lists so the elements are mutable
        key_copy = [ord(c) for c in key]
        enc_copy = [ord(c) for c in encoded]
        # Start with the last two bytes in the key
        var1 = key_copy[0x100]
        var2 = key_copy[0x101]
        # Do the RC4 algorithm
        for i in range(0, len(enc_copy)):
            var1 += 1
            a = var1 & 0xFF
            b = key_copy[a]
            var2 += b
            var2 &= 0xFF
            key_copy[a]  = key_copy[var2]
            key_copy[var2] = b
            enc_copy[i] ^= key_copy[(key_copy[a] + b) & 0xFF]
        # Return the decoded bytes as a string
        decoded = [chr(c) for c in enc_copy]
        return ''.join(decoded)

    def rc4_init(self, data):
        """Initialize the RC4 keystate"""
        # The key starts off as a mutable list
        key = list()
        for i in range(0, 256):
            key.append(i)
        # Add the trailing two bytes
        key.append(0)
        key.append(0)
        # Make a copy of the data so its mutable also
        data_copy = [ord(c) for c in data]
        var1 = 0
        var2 = 0
        for i in range(0, 256):
            a = key[i]
            var2 += (data_copy[var1] + a)
            var2 &= 0xFF
            var1 += 1
            key[i] = key[var2]
            key[var2] = a
        # Return a copy of the key as a string
        return ''.join([chr(c) for c in key])

    def decode_config(self, encoded_config, last_sec_data):
        """Decode the config with data from the last PE section. 

        @param encoded_config: the encoded configuration
        @param last_sec_data: last PE section data. 
        """

        return ''.join([chr(ord(last_sec_data[i]) ^ ord(encoded_config[i])) 
                        for i in range(len(encoded_config))])

    def check_matches(self, task_space, vad, matches, last_sec_data):
        """Check the Yara matches and derive the encoded/decoded 
        config objects and magic structures. 

        @param task_space: the process AS
        @param vad: the containing MMVAD 
        @param matches: list of YARA hits 
        @param last_sec_data: buffer of the last PE section's data
        """

        hits = dict((m.rule, m.strings[0][0] + vad.Start) for m in matches)

        ## Do the magic 
        if 'z3' in hits:
            addr = obj.Object('unsigned long', offset = hits['z3'] + 30, vm = task_space)
            size = task_space.profile.get_obj_size(self.magic_struct)
            self.params['encoded_magic'] = task_space.read(addr, size)
        elif 'z4' in hits:
            addr = obj.Object('unsigned long', offset = hits['z4'] + 31, vm = task_space)
            size = task_space.profile.get_obj_size(self.magic_struct)
            self.params['encoded_magic'] = task_space.read(addr, size)
        else:
            return False 

        ## Do the config 
        if 'z1' in hits:
            addr = obj.Object('unsigned long', offset = hits['z1'] + 8, vm = task_space)
            size = obj.Object('unsigned long', offset = hits['z1'] + 2, vm = task_space)
            encoded_config = task_space.read(addr, size)
            self.params['decoded_config'] = self.decode_config(encoded_config, last_sec_data)
        elif 'z2' in hits:
            addr = obj.Object('Pointer', offset = hits['z2'] + 26, vm = task_space)
            encoded_config = task_space.read(addr.dereference(), 0x3c8)
            rc4_init = self.rc4_init(encoded_config)
            self.params['decoded_config'] = self.rc4(rc4_init, last_sec_data[2:])
        elif 'z5' in hits:
            addr = obj.Object('unsigned long', offset = hits['z5'] + 8, vm = task_space)
            size = obj.Object('unsigned long', offset = hits['z5'] + 2, vm = task_space)
            encoded_config = task_space.read(addr, size)
            self.params['decoded_config'] = self.decode_config(encoded_config, last_sec_data)
        else:
            return False 

        ## We found at least one of each category 
        return True
    
    def decode_magic(self, config_key):
        """Decode the magic structure using the configuration key. 
        
        @param config_key: the config RC4 key.
        """

        return self.rc4(config_key, self.params['encoded_magic'])

    def scan_key(self, task_space):
        """Find the offset of the RC4 key and use it to 
        decode the magic buffer. 

        @param task_space: the process AS
        """

        offset = 0
        found = False

        while offset < len(self.params['decoded_config']) - RC4_KEYSIZE:

            config_key = self.params['decoded_config'][offset:offset + RC4_KEYSIZE]
            decoded_magic = self.decode_magic(config_key)

            # When the first four bytes of the decoded magic buffer 
            # equal the size of the magic buffer, then we've found 
            # a winning RC4 key
            (struct_size,) = struct.unpack("=I", decoded_magic[0:4])

            if struct_size == task_space.profile.get_obj_size(self.magic_struct):
                found = True
                self.params['config_key'] = config_key
                self.params['decoded_magic'] = decoded_magic
                break

            offset += 1

        return found 

    def calculate(self):
        
        if not has_yara:
            debug.error("You must install yara")

        addr_space = utils.load_as(self._config)

        if not self.is_valid_profile(addr_space.profile):
            debug.error("This command does not support the selected profile.")

        rules = yara.compile(sources = self.signatures)

        for task in self.filter_tasks(tasks.pslist(addr_space)):
            task_space = task.get_process_address_space()

            # We must have a process AS
            if not task_space:
                continue 

            for vad, process_space in task.get_vads(): 
            
                if obj.Object("_IMAGE_DOS_HEADER", offset = vad.Start, 
                        vm = process_space).e_magic != 0x5A4D:
                    continue
                    
                # a zeus range will never be more than 5 MB
                if vad.Length > 0x500000:
                    continue

                data = process_space.zread(vad.Start, vad.Length)
            
                # check for the signature with YARA, both hits must be present
                matches = rules.match(data = data)

                if len(matches) < 2:
                    continue

                try:
                    dos_header = obj.Object("_IMAGE_DOS_HEADER", 
                                    offset = vad.Start, vm = task_space)
                    nt_header = dos_header.get_nt_header()
                except (ValueError, exceptions.SanityCheckException):
                    continue 

                # There must be more than 2 sections 
                if nt_header.FileHeader.NumberOfSections < 2:
                    continue

                # Get the last PE section's data 
                sections = list(nt_header.get_sections())
                last_sec = sections[-1]
                last_sec_data = task_space.zread(
                                    (last_sec.VirtualAddress + vad.Start), 
                                    last_sec.Misc.VirtualSize
                                    )
                                    
                success = self.check_matches(task_space, vad, matches, 
                                            last_sec_data)

                if not success:
                    continue 

                success = self.scan_key(task_space)

                if not success:
                    continue 

                yield task, vad, self.params

    def render_extra(self, outfd, task, vad, params):
        """Show any Zeus specific fields"""

        rc4_offset = task.obj_vm.profile.get_obj_offset(self.magic_struct, 'rc4key')
        creds_key = params['decoded_magic'][rc4_offset:rc4_offset + RC4_KEYSIZE]

        outfd.write("{0:<30} : \n{1}\n".format("Credential RC4 key", 
                "\n".join(
                ["{0:#010x}  {1:<48}  {2}".format(vad.Start + o, h, ''.join(c))
                for o, h, c in utils.Hexdump(creds_key)
                ])))

    def render_text(self, outfd, data):
        """Render the plugin's default text output"""
        
        for task, vad, params in data:

            # Get a magic object from the buffer
            buffer_space = addrspace.BufferAddressSpace(
                                config = self._config, 
                                data = params['decoded_magic'])

            magic_obj = obj.Object(self.magic_struct, 
                                offset = 0, vm = buffer_space)

            outfd.write("*" * 50 + "\n")
            outfd.write("{0:<30} : {1}\n".format("Process", task.ImageFileName))
            outfd.write("{0:<30} : {1}\n".format("Pid", task.UniqueProcessId))
            outfd.write("{0:<30} : {1}\n".format("Address", vad.Start))

            # grab the URLs from the decoded buffer
            decoded_config = params['decoded_config']
            urls = []
            while "http" in decoded_config:
                url = decoded_config[decoded_config.find("http"):]
                urls.append(url[:url.find('\x00')])
                decoded_config = url[url.find('\x00'):]
            for i, url in enumerate(urls):
                outfd.write("{0:<30} : {1}\n".format("URL {0}".format(i), url))

            outfd.write("{0:<30} : {1}\n".format("Identifier", 
                ''.join([chr(c) for c in magic_obj.guid if c != 0])))
            outfd.write("{0:<30} : {1}\n".format("Mutant key", magic_obj.guid_xor_key))
            outfd.write("{0:<30} : {1}\n".format("XOR key", magic_obj.xorkey))
            outfd.write("{0:<30} : {1}\n".format("Registry", 
                "HKEY_CURRENT_USER\\SOFTWARE\\Microsoft\\{0}".format(magic_obj.keyname)))
            outfd.write("{0:<30} : {1}\n".format(" Value 1", magic_obj.value1))
            outfd.write("{0:<30} : {1}\n".format(" Value 2", magic_obj.value2))
            outfd.write("{0:<30} : {1}\n".format(" Value 3", magic_obj.value3))
            outfd.write("{0:<30} : {1}\n".format("Executable", magic_obj.exefile))
            outfd.write("{0:<30} : {1}\n".format("Data file", magic_obj.datfile))

            outfd.write("{0:<30} : \n{1}\n".format("Config RC4 key", 
                    "\n".join(
                    ["{0:#010x}  {1:<48}  {2}".format(vad.Start + o, h, ''.join(c))
                    for o, h, c in utils.Hexdump(params['config_key'])
                    ])))

            self.render_extra(outfd, task, vad, params)

class CitadelScan1345(ZeusScan2):
    """Locate and Decrypt Citadel 1.3.4.5 Configs"""

    signatures = {
    'namespace1':'rule z1 {strings: $a = {8B EC 83 EC 0C 8A 82 ?? ?? ?? ?? 88 45 FE 8A 82 01 01 00 00 88 45 FD 8A 82 02 01 00 00 B9 ?? ?? ?? ?? 88 45 FF E8 ?? ?? ?? ??} condition: $a}',
    'namespace2':'rule z2 {strings: $a = {56 BA ?? ?? 00 00 52 68 ?? ?? ?? ?? 50 E8 ?? ?? ?? ?? 8B 0D ?? ?? ?? ?? 03 0D ?? ?? ?? ?? 8B F2 2B C8} condition: $a}',
    'namespace3':'rule z3 {strings: $a = {68 ?? ?? 00 00 8D 85 ?? ?? ?? ?? 50 8D 85 ?? ?? ?? ?? 50 E8 ?? ?? ?? ?? B8 ?? ?? 00 00 50 68 ?? ?? ?? ??} condition: $a}'
    }

    magic_struct = '_CITADEL1345_CONFIG'

    def rc4(self, key, encoded, login_key):
        """Perform a basic RC4 operation. 
        
        Same as Zeus, but with incorporation of
        a LOGIN_KEY value."""
        # Turn the buffers into lists so the elements are mutable
        key_copy = [ord(c) for c in key]
        enc_copy = [ord(c) for c in encoded]
        # Start with the last two bytes in the key
        var1 = key_copy[0x100]
        var2 = key_copy[0x101]
        var3 = 0
        login_key_len = len(login_key);
        # Do the RC4 algorithm
        for i in range(0, len(enc_copy)):
            var1 += 1
            a = var1 & 0xFF
            b = key_copy[a]
            var2 += b
            var2 &= 0xFF
            key_copy[a]  = key_copy[var2]
            key_copy[var2] = b
            enc_copy[i] ^= key_copy[(key_copy[a] + b) & 0xFF]
            enc_copy[i] ^= ord(login_key[var3])
            var3 += 1
            if (var3 == login_key_len):
                var3 = 0

        # Return the decoded bytes as a string
        decoded = [chr(c) for c in enc_copy]
        return ''.join(decoded)

    def decode_magic(self, config_key):
        """Decode the magic buffer using RC4 and 
        the LOGIN_KEY."""

        return self.rc4(config_key, self.params['encoded_magic'], 
                        self.params['login_key'])

    def check_matches(self, task_space, vad, matches, last_sec_data):
        """Check the Yara matches and derive the encoded/decoded 
        config objects and magic structures. 

        @param task_space: the process AS
        @param vad: the containing MMVAD 
        @param matches: list of YARA hits 
        @param last_sec_data: buffer of the last PE section's data
        """

        hits = dict((m.rule, m.strings[0][0] + vad.Start) for m in matches)

        if 'z1' in hits:
            addr = obj.Object('unsigned long', offset = hits['z1'] + 30, vm = task_space)
            self.params['login_key'] = task_space.read(addr, 0x20)
        else:
            return False

        if 'z2' in hits:
            addr = obj.Object('unsigned long', offset = hits['z2'] + 8, vm = task_space)
            size = obj.Object('unsigned long', offset = hits['z2'] + 2, vm = task_space)
            encoded_config = task_space.read(addr, size)
            self.params['decoded_config'] = self.decode_config(encoded_config, last_sec_data)
        else:
            return False

        if 'z3' in hits:
            addr = obj.Object('unsigned long', offset = hits['z3'] + 31, vm = task_space)
            size = task_space.profile.get_obj_size(self.magic_struct)
            self.params['encoded_magic'] = task_space.read(addr, size)
        else:
            return False

        return True

    def render_extra(self, outfd, task, vad, params):
        """Show Citadel specific fields"""

        aes_key = self.rc4(params['config_key'], 
                           hashlib.md5(params['login_key']).digest(),
                           params['login_key'])

        outfd.write("{0:<30} : {1}\n".format("Login key", params['login_key'].upper()))
        outfd.write("{0:<30} : {1}\n".format("AES key", str(aes_key).encode('hex').upper()))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.find_file  as linux_find_file
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class linux_enumerate_files(linux_common.AbstractLinuxCommand):
    """Lists files referenced by the filesystem cache"""

    def calculate(self):
        linux_common.set_plugin_members(self)

        for (_, _, file_path, file_dentry)in linux_find_file.linux_find_file(self._config).walk_sbs():
            inode = file_dentry.d_inode

            yield inode, inode.i_ino, file_path

    def unified_output(self, data):
        return TreeGrid([("Inode Address", Address), ("Inode Number", int), ("Path", str)],
                        self.generator(data))

    def generator(self, data):
        for inode, inum, path in data:
            yield (0, [Address(inode.v()), int(inum), str(path)])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Inode Address", "[addr]"), ("Inode Number", "25"), ("Path", "")])
        for inode, inum, path in data:
            self.table_row(outfd, inode, inum, path) 



# Volatility
# Copyright (C) 2009-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.crashinfo as crashinfo

class MachOInfo(crashinfo.CrashInfo):
    """Dump Mach-O file format information"""
    
    target_as = ['MachOAddressSpace']
        
    def render_text(self, outfd, data):
    
        header = data.get_header()
        
        outfd.write("Magic: {0:#x}\n".format(header.magic))
        outfd.write("Architecture: {0}-bit\n".format(data.bits))
        
        self.table_header(outfd, [("File Offset", "[addrpad]"), 
                                  ("Memory Offset", "[addrpad]"), 
                                  ("Size", "[addrpad]"), 
                                  ("Name", "")])
        
        for seg in data.segs:
            self.table_row(outfd, seg.fileoff, seg.vmaddr, seg.vmsize, seg.segname)
